/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package anthill.types;

import anthill.*;

/**
 *  This interface represents an utility data structure that can be
 *  used by ants to obtain information about the return path to the
 *  requestor nest.
 * 
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public interface ReturnPath
{

  /**
   *  Initializes this data structure. This method must be invoked
   *  at most once, for example in the <CODE>init</CODE> method of
   *  ants.
   *  <BR>
   *  The implementation may make use of <CODE>view</CODE>, for 
   *  example using local resource storages to maintain information 
   *  about the return path.
   *  
   *  @param view the current view associated to the ant using this
   *   data structure
   */
  public void init(AntView view);
  
  /**
   *  Adds a new element to the path. Returns true if this element
   *  already belongs to the path, false otherwise. Depending on
   *  the implementation, if the element already belongs to the
   *  path, the element may be added again or not.
   *  <BR>
   *  The implementation may make use of <CODE>view</CODE>, for 
   *  example using local resource storages to maintain information 
   *  about the return path.
   *  
   *  @param view the current view associated to the ant using this
   *   data structure
   */
  public boolean addLast(AntView view, NestId id);
  
  /**
   *  Removes the last element of the path and returns it to
   *  the invoker. Returns null if there are no more elements
   *  in the path.
   *  <BR>
   *  The implementation may make use of <CODE>view</CODE>, for 
   *  example using local resource storages to maintain information 
   *  about the return path.
   *  
   *  @param view the current view associated to the ant using this
   *   data structure
   */
  public NestId removeLast(AntView view);
  
  /**
   *  Return the current length of the path.
   */
  public int size();
  
  /**
   *  Return the current length of the path.
   */
  public int capacity();
    
} // END ReturnPath
