/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.freeant;

import java.io.*;
import java.util.*;

import anthill.factory.Factory;
import anthill.util.Util;
import antsim.ResourceSet;

 /**
   * 
   * @author Alberto Montresor
   * @version $Revision: 1.2 $
   */
public class GnutantResourceSet
implements ResourceSet
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameter identifiers
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  String name of the parameter used to determine the number of
   *  of resources contained in this resource set.
   */
  public static final String PAR_SIZE = "Size";
  
  /**
   *  String name of the parameter used to determine the 
   */
  public static final String PAR_FILENAME = "Filename";
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Parameters
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Keywords filename */
  private String kwFile;

  /** Number of resources contained in this resource set */
  private int size;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Factory object used to read configuration parameters and classes */
  private Factory factory;

  /** List of keywords corresponding to resources in this resource set */
  private ArrayList keywords;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * 
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public GnutantResourceSet(Factory factory, String name)
  {
    this.factory = factory;
    clear();
  }

  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    kwFile = factory.stringValue(NAME + "." + PAR_FILENAME);
    size = factory.intValue(NAME + "." + PAR_SIZE);
    try {
      BufferedReader br = new BufferedReader(new FileReader(kwFile));
      keywords = new ArrayList(size);
      try {
        String keyline = null;
        /* Get the next line of keywords from the keyword file */
        while (((keyline = br.readLine()) != null) && (keywords.size() < size))
          keywords.add(keyline);
      } catch (IOException e) {
        /* Serious failure */
        Util.panic(e);
      }
    } catch (java.io.FileNotFoundException e) {
      /* Well, I suppose we should abort. */
      Util.panic(e);
    }
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * 
   */
  public List getResources()
  {
    return keywords;
  }
  
  
} // END GnutantResourceSet

