/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.freeant;

import java.io.Serializable;

import anthill.*;

/**
 *  In this version, I've removed the support for fake probability. Could be re-inserted
 *  later.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class ResponseGnutant
implements Ant, Serializable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The identifier of the request */
  private RequestId rid;

  /** Resources to be returned to the requestor nest */
  private Resource[] resources;
  
  /** Identifier of the requestor nest */
  private NestId requestor;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Initializes a new ResponseGnutant with the set of resources to
   *  be returned, the identifier of the requestor nest and an ant
   *  identifier to be used to obtain return information through nests.
   */
  public ResponseGnutant(RequestId rid, Resource[] resources, NestId requestor)
  {
    this.rid = rid;
    this.resources = resources;
    this.requestor = requestor;
  }
  
  /**
   *  Default constructor for externalization
   */
  public ResponseGnutant()
  {
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  // Documentation inherited by interface
  public String getSpecies()
  {
    return "Gnutant";
  }

  public void init(AntView view)
  {
  }

  /**
   *  This method encapsulate the computation performed by the
   *  ant. 
   *
   *  @param view reference to the AntView object which
   *    constitutes the controlled environment of the ant
   */
  public void run(AntView view)
  {
    if (requestor != null) {
      /*
       *  In this case, we are storing the identifier of
       *  the original requestor; this happens when the
       *  ant is created
       */
      NestId req = requestor;
      requestor = null;
      try {
        view.move(req);
      } catch (Exception e) {
        // Do nothing; the requestor is not rechable any more
      }
    } else {
      /*
       *  We have reached the requestor nest; deliver the responses.
       */
      view.returnResources(rid, resources);
    }
  }

} // END ResponseGnutant

