/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.io.Serializable;
import java.util.List;

import anthill.*;


/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class DeliverAnt
implements Ant, Serializable
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Tasks to be delivered in the local storage.
   */
  List tasks;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * 
   */
  public DeliverAnt(List tasks)
  {
    this.tasks = tasks;
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Ant
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Returns the identifier of the "species" of this ant.
   *  Different ant species may have access to different
   *  pheromone information in nests.
   */
  public String getSpecies()
  {
    return "MessorAnt";
  }

  public void init(AntView view)
  {
  }

  /**
   *  This method encapsulate the computation performed by the ant. 
   *  It obtains a reference to the task storage and inserts the
   *  tasks in it.
   *
   *  @param view reference to the AntView object which
   *    constitutes the controlled environment of the ant
   */
  public void run(AntView view)
  {
    TaskStorage storage = 
      (TaskStorage) view.createResourceManager(TaskStorage.NAME);
    storage.addTasks(tasks);
  }

} // END DeliverAnt

