/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.util.List;

import anthill.factory.Factory;
import anthill.nest.Nest;
import antsim.*;


/**
 *  A resource distribution for the messor ant species is empty, since 
 *  for messors there is no fixed set of "resources" in the system.
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.5 $
 */
public class MessorResourceDistribution
implements ResourceDistribution
{

  /**
   *  Constructs a "do-nothing" resource distribution.
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public MessorResourceDistribution(Factory factory, String name)
  {
  }
  
  /**
   *  In this implementation, this method does nothing.
   */
  public void distributeResources(ResourceSet resources, Peernet peernet)
  {
    /* Install a queen ant in each of the nests */
    // FIXME: This is not the natural place where to put this kind of behavior;
    // but unfortunately, I can't see any other place where to put it currently.
    // Instead of adding new interfaces to the current simulator, we will wait the
    // new simulator
    List nests = peernet.getNests();
    for (int i=0; i < nests.size(); i++) {
      Nest nest = (Nest) nests.get(i);
      nest.request(new Rid(nest), new Object(), null); // FIXME We should create the correct request
    }
  }

  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
  }
  
} // END MessorResourceDistribution

