/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.util.*;

import anthill.factory.Factory;
import anthill.nest.Nest;
import anthill.util.Log;
import antsim.*;

/**
 *  This class implements the <CODE>ScenarioAnalyzer</CODE> interface.
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class NestNeighboursAnalyzer
implements ScenarioAnalyzer
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Used to print headers */
  static private boolean firstTime = true;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Instantiates a new NestNeighboursAnalyzer. 
   *
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public NestNeighboursAnalyzer(Factory factory, String name)
  {
    clear();
  }
  
  
  /**
   * Restores the object to its initial state.
   */
  public void clear()
  {
  }

    
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Performs an analysis on the state of the simulated P2P network.
   *  
   *  @param stats the object used to collect statistics
   *  @param peernet the simulated P2P network, whose content can
   *    be inspected by accessing the storage objects included in it.
   */
  public void analyze(Stats stats, Peernet peernet)
  {
    if (firstTime) {
      firstTime = false;
      Log.println(200, "Round ; Reachable nests ");
    }

    /* Number of ids present in the system */
    Set ids = new HashSet();
    
    List nests = peernet.getNests();
    for (int k=0; k < nests.size(); k++) {
      Nest nest = (Nest) nests.get(k);
      MessorPheromone pheromone = (MessorPheromone)
        nest.createResourceManager(MessorPheromone.NAME, null);
      PheromoneEntry[] table = pheromone.getEntryTable();
      for (int i=0; i < table.length; i++) {
        if (table[i] != null) {
          ids.add(table[i].getNestId());
          // System.out.println(table[i].getNestId() + ".load = "+table[i].getLoad());
        }
      }
      // System.out.println("--------------------------------------");
    } 
  
    StringBuffer buffer = new StringBuffer();
    buffer.append( (int) stats.get("Iteration"));
    buffer.append(" ; ");
    buffer.append(ids.size());
    Log.println(200, buffer.toString());
  }
  
} // END NestNeighboursAnalyzer

