/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import anthill.*;

/**
 *  For the purpose of simulation, a task does not perform anything.
 *  It just implements a request identifier. In this simple implementation,
 *  all tasks are equal: i.e., they last
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public class SimulatedTask
implements Task, RequestId
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static section
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Counter used to generate progressive ids for tasks */
  private static int counter = 0;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /** Progressive id to identify distinct tasks */
  int id;
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public SimulatedTask()
  {
    id = counter++;
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Method to be invoked to execute this task.
   */
  public void run()
  {
  }

  /**
   * Returns the load associated to this task.
   */
  public int getLoad()
  {
    return 1;
  }
  

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from RequestId
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Unsupported operation
   */
  public NestId getNestId()
  {
  	throw new UnsupportedOperationException();
  }
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Object
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Check whether this <CODE>SimulatedTask</CODE> is equal to the specified 
   * object.
   */
  public boolean equals(Object obj)
  {
    if (obj != null && obj instanceof SimulatedTask) {
      SimulatedTask task = (SimulatedTask) obj;
      return (task.id == this.id);
    } else {
      return false;
    }
  }
  
  /**
   * Returns a hash code value for the object.
   */
  public int hashCode()
  {
    return id;
  }
  
  /**
   * Returns a string representation of the object.
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append("[Task: ");
    buf.append(id);
    buf.append("]");
    return buf.toString();
  }
  
} // END SimulatedTask

