/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package ants.messor;

import java.util.*;

import anthill.factory.Factory;

/**
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.5 $
 */
public class SimulatedTaskStorage
implements TaskStorage, EventListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Task set */
  private Set tasks;

  /** Current task in execution */
  private Task currentTask;
  
  /** Current task load included in the simulated storage */
  private int load;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public SimulatedTaskStorage(Factory factory, String name)
  {
    tasks = new HashSet();
    currentTask = null;
    load = 0;
  }
  
  public void clear()
  {
    tasks.clear();
    currentTask = null;
    load = 0;
  }
 
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods 
  ////////////////////////////////////////////////////////////////////////////////////////////
  
  /**
   *  Add the task to the task set.
   */
  public int addTasks(List list)
  {
    for (int i=0; i < list.size(); i++) {
      Task task = (Task) list.get(i);
      if (tasks.add(task)) {
        load = load + task.getLoad();
      }
    }
    return 0; // FIXME
  }
  
  /**
   *  Get tasks from the task set
   */
  public List getTasks(int size)
  {
    ArrayList list = new ArrayList(size);
    Iterator iterator = tasks.iterator();
    int i = 0;
    while (iterator.hasNext() && i < size) {
      Task task = (Task) iterator.next();
      list.add(task);
      load = load - task.getLoad();
      iterator.remove();
      i++;
    }
    return list;
  }

  /**
   *  Returns the current load of this task storage.
   */
  public int getLoad()
  {
    return load;
  }

  /**
   *  This method will be used to notify the storage that a certain
   *  event has occurred.
   */
  public void event(Object obj)
  {
    System.out.println("+");
    // FIXME. The idea is that to write an event-based simulator,
    // in which events are delivered to the system when they occur.
    // Currently, we will use it to notify rounds
  }
  
} // END SimulatedTaskStorage


