/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim;

import anthill.util.Clearable;

/**
 *  Interface <CODE>ScenarioAnalyzer</CODE> represents analyzer object
 *  that are invoked at the end of each scenario evaluation. The main
 *  task of an analyzer object is to display insightful data aboyt the
 *  simulation. Different implementation may display different set of
 *  datas; we expect that each type of ant service will be associated
 *  with a distinct analyzer.
 *  
 *  @author Alberto Montresor
 *  @version $Revision: 1.2 $
 */
public interface ScenarioAnalyzer
extends Clearable
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Identifier used in the configuration files */
  public static final String NAME = "ScenarioAnalyzer";
 
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Performs an analysis on the state of the simulated P2P network.
   *  Depending on the implementation of this interface, the analysis
   *  may involve the statistics collected during the run-time evaluation
   *  of the system, or the snapshot of the simulated network represented
   *  by the <CODE>Peernet</CODE> object.
   *  
   *  @param stats the object used to collect statistics
   *  @param peernet the simulated P2P network, whose content can
   *    be inspected by accessing the storage objects included in it.
   */
  public void analyze(Stats stats, Peernet peernet);
  
} // END ScenarioAnalyzer

