/*
 * Copyright (c) 2001 The Anthill Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package antsim.nestor;

import java.util.HashMap;

import anthill.*;
import anthill.factory.Factory;
import anthill.nest.*;

/**
 *  This class is a simple implementation of <CODE>ResourceSet</CODE>
 *  containing an hash table mapping ant identifiers to response
 *  listeners.
 *
 *  @author Alberto Montresor
 *  @version $Revision: 1.3 $
 */
public class LocalRequestSet
implements RequestSet
{
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Map that associates ant identifiers to response listeners */
  HashMap map;
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructor
   *  
   *  @param factory the factory to be used to read configuration arguments
   *    and to create other configurable objects
   *  @param name the name that is associated to this instance in the
   *    configuration file
   */
  public LocalRequestSet(Factory factory, String name)
  {
    map = new HashMap();
  }
  
  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods 
  ////////////////////////////////////////////////////////////////////////////////////////////

  // Comment inherited from interface RequestSet
  public void add(RequestId key, ResponseListener listener)
  {
    map.put(key, listener);
  }
  
 
  // Comment inherited from interface RequestSet
  public boolean returnResources(RequestId key, Resource[] resources)
  {
    ResponseListener listener = (ResponseListener) map.get(key);
    if (listener != null) {
      listener.returnResources(key, resources);
      return true;
    }
    return false;
  }

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Clearable
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Restores the object to its initial state.
   */
  public void clear()
  {
    map.clear();
  }

  
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    buffer.append("[ReqSet Size: ");
    buffer.append(map.size());
    buffer.append("]");
    return buffer.toString();
  }
  

} // END LocalRequestSet
