/******************************************************
 * File: DowloadManager.java
 * created 06-Mar-02 9:57:12 AM by root
 */


package jxtaimpl;

import java.io.*;
import java.net.*;

import net.jxta.document.AdvertisementFactory;
import net.jxta.endpoint.*;
import net.jxta.id.IDFactory;
import net.jxta.pipe.*;
import net.jxta.protocol.PipeAdvertisement;

/**
*  This class manages uploads required by remote nests. The JXTA-Enabled 
*  <code>DownloadService</code> instantiates this class for each incoming request 
*  @author Francesco Russo (frusso@cs.unibo.it)
*/
public class DownloadManager extends Thread
{
	/**
	*  This is the constructor.
	*
	* @param nest The local nest <code>jxtaimpl.Nest</code> instance
	* @param pipeServ A reference to the <code>JXTA-PipeService</code> instance
	* @param msg The <code>net.jxta.endpoint.Message<code> message to process
	*/
	public DownloadManager(Nest nest, PipeService pipeServ, Message msg)
	{
		localNest = nest;
		message = msg;
		pipeService = pipeServ;
	}
	
	/**
	*  The DownloadManager's logic
	*/
	public void run()
	{
		try
		{
			byte[] id = message.getBytes( PIPEID );
			byte[] name = message.getBytes( PIPENAME );
			byte[] type = message.getBytes( PIPETYPE );
			byte[] resId = message.getBytes( RESOURCEID );
			byte[] species = message.getBytes( SPECIES );
				
			PipeAdvertisement outPipeAdv = (PipeAdvertisement)AdvertisementFactory.newAdvertisement( PipeAdvertisement.getAdvertisementType() );
			outPipeAdv.setName( new String(name) );
			outPipeAdv.setPipeID( IDFactory.fromURL(new URL(new String(id))) );
			outPipeAdv.setType( new String(type) );
			
			OutputPipe outPipe = pipeService.createOutputPipe( outPipeAdv, WAITTIME );
			
			InputStream is = localNest.getInputStreamForResource( resId, species );
			
			Message outMsg = pipeService.createMessage();
			MessageElement outMsgEl = outMsg.newMessageElement( RESOURCE, null, is );
			outMsg.addElement( outMsgEl );
			
			outPipe.send( outMsg );
		}
		catch(MalformedURLException mue)
		{
			mue.printStackTrace();
		}
		catch(UnknownServiceException use)
		{
			use.printStackTrace();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}
	}
	
	private Message message;
	private Nest localNest;
	private PipeService pipeService;
	
	private static final long WAITTIME = 10000;
	
	//* tags for incoming requests for downloading resources
	private static final String PIPEID = "PIPEID";
	private static final String PIPENAME = "PIPENAME";
	private static final String PIPETYPE = "PIPETYPE";
	private static final String RESOURCEID = "RESOURCEID";
	private static final String SPECIES = "SPECIES";
	
	//* tag for outgoing messages for uploading resources
	private static final String RESOURCE = "RESOURCE";
}
