/******************************************************
 * File: DownloadService.java
 * created 05-Mar-02 7:49:02 PM by root
 */


package jxtaimpl;

import java.io.IOException;

import net.jxta.document.Advertisement;
import net.jxta.endpoint.Message;
import net.jxta.id.ID;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.*;
import net.jxta.protocol.*;
import net.jxta.service.Service;

/**
*  This class is a JXTA-Enabled Service. Its task is waiting for incoming requests
*  for uploading resources originated by remote nests. 
*  @author Francesco Russo (frusso@cs.unibo.it)
*/
public class DownloadService extends Thread
							 implements Service
{
	/**
	*  Initialization of this service's parameters
	*/
	public void init(PeerGroup grp, ID id, Advertisement adv) 
  	{	
		anthillGroup = ((AnthillGroup)grp);
		pipeService = ((AnthillGroup)grp).getNetPipeService();
		localNest = ((AnthillGroup)grp).getNest();
		dsPipeAdv = ((AnthillGroup)grp).getDsPipeAdv();
		downloadServiceAdvertisement = (ModuleImplAdvertisement) adv;
		try
		{
			inPipe = pipeService.createInputPipe( dsPipeAdv );
		}
		catch(IOException ioe)
		{
			System.out.println("--- DownloadService: init() - IOException while creating InputPipe");
		}
		System.out.println("--- DownloadService initialized");
	}
	
	/**
	*  Starting method
	*/
	public int startApp(String[] args) 
  	{
		this.start();
		System.out.println("--- DownloadService started!");
		return( 0 );
	}
	
	public void stopApp() 
  	{
  	}

	public Service getInterface()
  	{
		return this;
  	}
	
	public Advertisement getImplAdvertisement() 
  	{
		return downloadServiceAdvertisement;
  	}
	
	/**
	*  For each message received, a <code>DownloadManager</code> is instantiated
	*  and started.
	*/
	public void run()
	{
		while(true)
		{
			try
			{
				Message msg = inPipe.waitForMessage();
				System.out.println("--- DownloadService: waitForMessages() message received!");
				DownloadManager dMng = new DownloadManager( localNest, pipeService, msg );
				System.out.println("--- DownloadService: waitForMessages() created the DownloadManager!");
				dMng.start();
				System.out.println("--- DownloadService: waitForMessages() started!");
			}
			catch(InterruptedException ie)
			{
				System.out.println("--- DownloadService: waitForMessages() InterruptedException!");
				ie.printStackTrace();
			}
		}
	}
	
	private ModuleImplAdvertisement downloadServiceAdvertisement;
	private PipeService pipeService;
	private InputPipe inPipe;
	private PipeAdvertisement dsPipeAdv;
	private AnthillGroup anthillGroup;
	private Nest localNest;
}
