/* Generated by Together */

package jxtaimpl;

import java.io.*;

import net.jxta.endpoint.Message;
import net.jxta.pipe.InputPipe;
import net.jxta.protocol.PipeAdvertisement;

/**
 * This class holds the input pipe for the GateService. All the incoming messages have to be saved in the GateServiceBuffer, and not processed. 
 * @author Francesco Russo (frusso@cs.unibo.it)
 * @stereotype thread
 */
public class GateServiceReceiver extends Thread 
{
  /**
  * This is the default constructor
  */
  public GateServiceReceiver() 
  {
    super();
    this.setName( NAME );
  }

  /**
   * This is the buffer used to handle all the incoming messages to the GateServiceServer 
   */
  public void setBuffer(GateServiceBuffer buffer) 
  {
    gsBuffer = buffer;
  }
  
  /**
  * This method is invoked by the GateService instance during the initialization process in order to pass this
  * object the tag to look for in all the incoming messages. That'a the tag under which an Ant is sent
  */
  protected void setMsgAntTag(String tag)
  {
    MSG_ANT_TAG = tag;
  }

  public void run() 
  {
    while (true)
    {
      receiveMessages();
    }
  }

  /**
   * This method is used for passing a PipeAdvertisement to the GateServiceReceiver instance. The input pipe will be generated from this PipeAdvertisement. 
   */
  public void setInputPipeAdv(PipeAdvertisement pipeAdv) 
  {
    inputPipeAdv = pipeAdv; 
  }

  /**
  * This method is used by the GateServiceInstance for setting the input pipe from which messages should come
  */
  public void setInputPipe(InputPipe pipe)
  {
    inPipe = pipe;
  }
  
  /**
  * In this method resides receiver's logic
  */
  private void receiveMessages()
  {
    try
    {
      msg = inPipe.waitForMessage();
      
      System.out.println("--- GateServiceReceiver: receiveMessage()");
    
      //* let's extract the AntDescriptor from the Message instance, and
      //* write it in the GateServiceBuffer in the shape of an Object
      buffer =  msg.getBytes( MSG_ANT_TAG );
      bais = new ByteArrayInputStream( buffer );
      ois = new ObjectInputStream( bais );
      Object obj = ois.readObject();          
      
      gsBuffer.write( obj );
    }
    catch(InterruptedException ie)
    {
      ie.printStackTrace();
      System.exit(1);
    }
    catch(IOException ioe)
    {
      ioe.printStackTrace();
      System.exit(1);
    }
    catch (ClassNotFoundException cnfe)
    {
      cnfe.printStackTrace();
      System.exit(1);
    }
  }

  private GateServiceBuffer gsBuffer;
  private PipeAdvertisement inputPipeAdv;
  private InputPipe inPipe;
  private Message msg;
  private byte[] buffer;
  private ByteArrayInputStream bais;
  private ObjectInputStream ois;
  private String NAME = "GateServiceReceiver-Thread";
  private String MSG_ANT_TAG;
}
