/* Generated by Together */

package jxtaimpl;

import java.io.*;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.*;

import jxtaimpl.storage.*;

import net.jxta.platform.ModuleClassID;
import net.jxta.protocol.PipeAdvertisement;

import anthill.*;
import anthill.factory.*;
import anthill.nest.*;
import anthill.storage.*;
import ants.utils.InsantRequest;

/**
 * This class implements the Nest interface and the AntListener interface as well. 
 * A GateServiceServer will notify its Nest instance upon receiving a new message 
 * containing an Ant via the method deliver(AntDescriptor).
 *
 * @author Francesco Russo (frusso@cs.unibo.it)
 * @stereotype implementationClass
 */
public class Nest implements AntListener, anthill.nest.Nest
{
  /**
  * Default constructor which instantiates a RequestSet and a Trail
  *
  * @param fact The <code>Factory</code> instace useful for reading in configuration parameters
  */
 public Nest( Factory fact ) 
 {
  factory = fact;
  saveInfo = factory.stringValue( SERIALIZATION );
  //confDir = factory.stringValue( CONF_DIR );
  
  if ( saveInfo.trim().toLowerCase().equals("yes") )
  {
   if ( anthillDir == null )
   {  
	anthillDir = new File(System.getProperty("user.dir")+File.separator+CMS_DIR_NAME);
	if ( !anthillDir.exists() )
	{
	  //* the AnthillResources folder doesn't exists, let's create it
	  anthillDir.mkdir();
	  filePheromoneHashTable = new File( anthillDir, PHEROMONE_H_TAB_FILE_NAME );
	  fileURLHashTable = new File( anthillDir, URL_H_TAB_FILE_NAME );
	  try
	  {
	    filePheromoneHashTable.createNewFile();
	    FileOutputStream fos = new FileOutputStream( filePheromoneHashTable );
	    ObjectOutputStream oos = new ObjectOutputStream( fos );
	    pheromoneHashTable = new Hashtable();
	    oos.writeObject( pheromoneHashTable );
	    fos.close();
		
		fileURLHashTable.createNewFile();
	    fos = new FileOutputStream( fileURLHashTable );
	    oos = new ObjectOutputStream( fos );
	    URLHashTable = new Hashtable();
	    oos.writeObject( URLHashTable );
	    fos.close();
	  }
	  catch(IOException ioe)
	  {
	    ioe.printStackTrace();
	    System.out.println("--- Nest: could not create configurationfiles, giving up!");
	    System.exit(1);
	  }
	}
	else
	{
	  //* the AnthillResources folder exists, so I've to check whether the 
	  //* configuration files are there or not
	  String[] files = anthillDir.list();
	  boolean found = false;
	  boolean foundURL = false;
	  
	  for (int i=0; i < files.length; i++)
	  {
	    if ( files[i].trim().toLowerCase().equals(PHEROMONE_H_TAB_FILE_NAME.trim().toLowerCase()) )
	    {
  		  //* instantiate the pheromoneHashTable deserializing this file's content
		  try
		  {
		    filePheromoneHashTable = new File( anthillDir, PHEROMONE_H_TAB_FILE_NAME );
		    FileInputStream fis = new FileInputStream( filePheromoneHashTable );
		    ObjectInputStream ois = new ObjectInputStream( fis );
		    pheromoneHashTable = (Hashtable) ois.readObject();
		    fis.close();
		    found = true;
		    //break;
		  }
		  catch(IOException ioe)
		  {
		    ioe.printStackTrace();
		    System.out.println("--- Nest: could not read configuration file "+PHEROMONE_H_TAB_FILE_NAME+", giving up!");
		    System.exit(1);
		  }
		  catch(ClassNotFoundException cnfe)
		  {
		    cnfe.printStackTrace();
		    System.out.println("--- Nest: could not read configuration file "+PHEROMONE_H_TAB_FILE_NAME+", giving up!");
		    System.exit(1);
		  }
	    }
	    else
		{
		  if ( files[i].trim().toLowerCase().equals(URL_H_TAB_FILE_NAME.trim().toLowerCase()) )
	      {
   		    //* instantiate the URLHashTable deserializing this file's content
		    try
		    {
  		      fileURLHashTable = new File( anthillDir, URL_H_TAB_FILE_NAME );
		      FileInputStream fis = new FileInputStream( fileURLHashTable );
		      ObjectInputStream ois = new ObjectInputStream( fis );
		      URLHashTable = (Hashtable) ois.readObject();
		      fis.close();
		      foundURL = true;
		      //break;
		    }
		    catch(IOException ioe)
		    {
  		      ioe.printStackTrace();
		      System.out.println("--- Nest: could not read configuration file "+URL_H_TAB_FILE_NAME+", giving up!");
		      System.exit(1);
		    }
		    catch(ClassNotFoundException cnfe)
		    {
		      cnfe.printStackTrace();
		      System.out.println("--- Nest: could not read configuration file "+URL_H_TAB_FILE_NAME+", giving up!");
		      System.exit(1);
		    }
	      }  
		}
	  }
	  if ( found==false )
	  {
	    //* the pheromone file does not exists, let's create it
	    filePheromoneHashTable = new File( anthillDir, PHEROMONE_H_TAB_FILE_NAME );
	    try
	    {
		  filePheromoneHashTable.createNewFile();
		  FileOutputStream fos = new FileOutputStream( filePheromoneHashTable );
		  ObjectOutputStream oos = new ObjectOutputStream( fos );
		  pheromoneHashTable = new Hashtable();
		  oos.writeObject( pheromoneHashTable );
		  fos.close();
	    }
	    catch(IOException ioe)
	    {
  		  ioe.printStackTrace();
		  System.out.println("--- Nest: could not create configurationfiles, giving up!");
		  System.exit(1);
	    }
	  }
	  if ( foundURL==false )
	  {
	    //* the URL file does not exists, let's create it
	    fileURLHashTable = new File( anthillDir, URL_H_TAB_FILE_NAME );
	    try
	    {
		  fileURLHashTable.createNewFile();
		  FileOutputStream fos = new FileOutputStream( fileURLHashTable );
		  ObjectOutputStream oos = new ObjectOutputStream( fos );
		  URLHashTable = new Hashtable();
		  oos.writeObject( URLHashTable );
		  fos.close();
	    }
	    catch(IOException ioe)
	    {
  		  ioe.printStackTrace();
		  System.out.println("--- Nest: could not create configurationfiles, giving up!");
		  System.exit(1);
	    }
	  }
	}
   }
   else
   {
	  System.out.println("--- Nest: only one nest per user is permitted!");
	  System.exit(1);
	}
  }
  else
  {  
	//* the configuration file says we have not to read the objects
	//* from file
	pheromoneHashTable = new Hashtable();
	URLHashTable = new Hashtable();
  }
 }

  /**
   * Method present in the Interface declaration, but not need and so not 
   * implemented in the <CODE>jxtaimpl</CODE> implementation.
   */
  public void leave()
  {
  }

  /**
   * Restores the Nest instance to its initial state
   */
  public void clear()
  {
	reqSet = new RequestSet();
	trail = new Trail();
  
	DOCstorageHashTable = new Hashtable();
	String knownSpecies = factory.stringValue( KNOWN_SPECIES.trim() );
	StringTokenizer sTok = new StringTokenizer( knownSpecies );  
	while( sTok.hasMoreElements() )
	{
	  String species = ((String)sTok.nextElement()).trim().toLowerCase();
	  int size = factory.intValue(species);
	  CmsStorage storage = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
	  DOCstorageHashTable.put(species, storage);
	}
  
	fileNeighbourManager = new File( anthillDir, NEIGHBOUR_MNG_FILE_NAME );
   }
  
  /**
   *  Stores all the information useful for locating this nest's input pipe
   *  waiting for resource downloading requests. 
   */
  protected void setDownloadServicePipeAdv(PipeAdvertisement pipeAdv)
  {
	downloadPipeId = pipeAdv.getPipeID().getURL();
	downloadPipeName = pipeAdv.getName();
	downloadPipeType = pipeAdv.getType();
  
	clear();
  
	//System.out.println("Nest: "+downloadPipeId+" "+downloadPipeName+" "+downloadPipeType);
  }
  
  /**
  * Method invoked by the AnthillPeerWS for setting the appropriate TTL value
  * which is read directly from the xml configuration file
  */
  protected void setTTL(int ttl)
  {
	TTL = ttl;
  }
  
  /**
  * Method invoked by the AnthillPeerWS for setting the appropriate GateServiceID value
  * which is needed in order to lookup the service itself.
  */
  protected void setGateServiceID( ModuleClassID id )
  {
	ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID = id; 
  }

  /**
  * Returns the local Nest id
  */
  public anthill.NestId getNestId() 
  { 
	return id;
  }

  /**
  * Returns a reference to the Storage instance associated with the 
  * invoking ant's species. If the storage didn't exist, <code>null</code>
  * would be returned.
  * Name is used for specifieing the storage type (i.e. Storage.DOC_STORAGE,
  * Storage.PHR_STORAGE, Storage.URL_STORAGE). At the moment 
  * Storage.DOC_STORAGE, Trail.NAME, Storage.PHR_STORAGE are supported.              
  *
  * @param name The type of <code>Storage</code>
  * @param species The ant species 
  */
  public ResourceManager getResourceManager(String name, String species) throws StorageNotFoundException  
  {
	ResourceManager resource = null;
	species = species.trim().toLowerCase();
  
	if ( name.equals(Storage.URL_STORAGE) )
	{
	  if( URLHashTable.containsKey(species) )
	  {
	    resource = (ResourceManager) URLHashTable.get(species);
	  }  
	}
	else
	{
	  if ( name.equals(Storage.DOC_STORAGE) )
	  {
 	    if ( DOCstorageHashTable.containsKey(species) )
	    {
	      resource = (ResourceManager)DOCstorageHashTable.get(species);
	    }
	  }
	  else
	  {
		if( name.equals(Trail.NAME) )
	    {
	      resource = (ResourceManager)this.getTrail( false ); 
	    }
	    else
	    {
		  if ( name.equals( Storage.PHR_STORAGE ) )
  	      {  
		    //* this should be a pheromone object
		    resource = (ResourceManager) this.getPheromone( species );
		  }
	    }
	  }
	}
	return resource;
  }
  
	// FIXME (x FRANCESCO): right semantics of this method to be implemented
	/**
	 * Returns a reference to the Storage instance associated with the 
	 * invoking ant's species. If the storage didn't exist, it would be created.
	 * Name is used for specifieing the storage type (i.e. Storage.DOC_STORAGE,
	 * Storage.PHR_STORAGE, Storage.URL_STORAGE). At the moment just the
	 * DOC_STORAGE is supported.
	 *
	 * @param name The type of <code>Storage</code>
	 * @param species The ant species 
	 */
	public ResourceManager createResourceManager(String name, String species) throws ResourceManagerException  
	{
		ResourceManager resource = null;
		species = species.trim().toLowerCase();
		
		if ( name.equals(Storage.URL_STORAGE) )
	    {
	      if ( URLHashTable.containsKey(species) )
		  {
		    resource = (ResourceManager) URLHashTable.get(species);
		  }
		  else
		  {
		    try
			{
			  resource = (ResourceManager) factory.get( name );
			  URLHashTable.put( species.trim().toLowerCase(), resource );
		    }
		    catch( ConfigFormatException cfe )
		    {
		      throw new ResourceManagerException(null);
		    }  
		  }
		}
		else
		{
		  if ( name.equals(Storage.DOC_STORAGE) )
		  {
			if ( DOCstorageHashTable.containsKey(species) )
			{
				resource = (ResourceManager)DOCstorageHashTable.get(species);
			}
			else
			{
				int size = factory.intValue(species);
				resource = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
				DOCstorageHashTable.put(species, resource);
			}
		  }
		  else
		  {
		    if ( name.equals(Trail.NAME) )
		    {
		       resource = (ResourceManager)this.getTrail(true);
		    }
		    else
		    {
			  if ( name.equals( Storage.PHR_STORAGE ) )
			  {
		        //* this should be a request for a pheromone object
			    resource = (ResourceManager) this.getPheromone( species );
			    if (resource == null)
			    {
			      //* instantiate the pheromone obj through the factory
			      try
			      {
			        resource = (ResourceManager) factory.get( name );
			        this.setPheromone( species, resource );
			      }
			      catch( ConfigFormatException cfe )
			      {
			        throw new ResourceManagerException(null);
			      }
			    }
			  }
		    }
		  }
		}
		
		return resource;
	}
	
	/**
	 * With this method a new request is passed to the Nest. 
	 * The <CODE>Object req</CODE> argument has to be a <CODE>jxtaimpl.CmsStorage.Query</CODE> 
	 * instance. If the request could be satisfied locally no Ants would be generated, otherwise 
	 * a set of ants will be sent across the network in order to satisfy
	 * the request. The response will be passed to the specified listener.
	 *
	 * @param rid The ID identifieing the request
	 * @param req The request object. It has to be a <code>jxtaimpl.storage.Query</code>
	 *        instance 
	 * @param lstn The listener to be notified when receiving replies
	 */
  public void request(anthill.RequestId rid, Object req, anthill.nest.ResponseListener lstn) 
  {
	reqSet.add( rid, lstn );
	Ant[] ants = antFactory.getAnts( rid, req, this.id );
	for (int i=0; i < ants.length; i++)
	{
	  AntDescriptor aDesc = new AntDescriptor( this.id, ants[i], this.TTL );
	  AntView aView = new AntView( this, aDesc, factory );
	  ants[i].init( aView );
	  manager.add( aView );
	}
  }
  
  /**
  * Method to be used by higher level applications for spreading the knowledge
  * about the local insertion of a new request.
  *
  * @param rid a valid request identifier
  * @param keywords a String of possible keywords for the resource
  */
  public void insRequest(anthill.RequestId rid, String keywords) throws IOException
  {
	InsantRequest insReq = new InsantRequest( downloadPipeId, keywords );
	
	Ant[] ants = antFactory.getAnts( rid, insReq, this.id );
	for (int i=0; i < ants.length; i++)
	{
	  AntDescriptor aDesc = new AntDescriptor( this.id, ants[i], this.TTL );
	  AntView aView = new AntView( this, aDesc, factory );
	  ants[i].init( aView );
	  manager.add( aView );
	}
  }

  /**
  * Add a new resource to this Nest. The resource has to be inserted into the local storage.
  * The system will look for a Storage instance associated with the 
  * invoking ant's species. If the storage didn't exist, it would be created.
  *
  * @param res The <code>jxtaimpl.storage.AnthillResource</code> instance describing the
  *        resource to store
  * @param species The ant species of the ant invoking this method
  */
  public void insert(anthill.Resource res, String species) throws CapacityException 
  {
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	GenericStorage storage = (GenericStorage)DOCstorageHashTable.get(species);
	synchronized (storage)
	{
	  storage.addResource( null, res );
	}
  }
  else
  {
	int size = factory.intValue(species);
	GenericStorage storage = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
	synchronized (storage)
	{
	  storage.addResource( null, res );
	  DOCstorageHashTable.put(species, storage);
	}
  }
  }
  
  /**
  * This method has to be used by higher level application in order to share 
  * a resource in the Anthill network.
  *
  * @param file The file containing the resource to share
  * @param name The resource logical name
  * @param type The resource type
  * @param description A logical description of the reaource
  * @param species The species of the invoking ant
  */
  public void shareResource( File file, String name, String type, String description, String species) throws SpeciesNotFoundException,
  																										     ResourceAlreadyStoredException,
																											 CapacityException
  {
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = (CmsStorage)DOCstorageHashTable.get(species);
	synchronized (storage)
	{
	  storage.share( file, name, type, description );
	}
  }
  else
  {
	try
	{
	  int size = factory.intValue(species);
	  CmsStorage storage = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
	  synchronized (storage)
	  {
	    storage.share( file, name, type, description );
	    DOCstorageHashTable.put(species, storage);
	  }
	}
	catch(IllegalArgumentException iae)
	{
	  throw new SpeciesNotFoundException(species);
	}
  }  
  }
  
  /**
  * This method has to be used by higher level application in order to share 
  * a resource in the Anthill network.
  *
  * @param file The <code>byte</code> array containing the resource to share
  * @param name The resource logical name
  * @param type The resource type
  * @param description A logical description of the reaource
  * @param species The species of the invoking ant
  */
  public void shareResource( byte[] file, String name, String type, String description, String species) throws SpeciesNotFoundException,
  																										     ResourceAlreadyStoredException,
																											 CapacityException
  {
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = (CmsStorage)DOCstorageHashTable.get(species);
	synchronized (storage)
	{
	  storage.share( file, name, type, description );
	}
  }
  else
  {
	try
	{
	  int size = factory.intValue(species);
	  CmsStorage storage = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
	  synchronized (storage)
	  {
	    storage.share( file, name, type, description );
	    DOCstorageHashTable.put(species, storage);
	  }
	}
	catch(IllegalArgumentException iae)
	{
	  throw new SpeciesNotFoundException(species);
	}
  }  
  }

  /**
  * This method will return all the resources locally stored in every 
  * storage. If nothing is found, <CODE>null</CODE> will be returned.
  */
  public Resource[] getAllResources()
  { 
   Enumeration enum = DOCstorageHashTable.elements();
   Vector vect = new Vector();
  
   while( enum.hasMoreElements() )
   {
	 CmsStorage storage = (CmsStorage) enum.nextElement();
	 if ( storage!= null )
	 {
	   Resource[] res = storage.getAllResources();
	   if (res!=null)
	   {
	     for(int i=0; i<res.length; i++)
	     {
	       vect.addElement( res[i] );
	     }
	   }
	 }
   }
   int size = vect.size();
   if ( size > 0 )
   {
	 Resource[] res = new Resource[ size ];
	 for(int i = 0; i<size; i++)
	 {
	   res[i] = (Resource)vect.get(i);
	 }
	 return res;
   } 
  return null;
  }

  /**
  * Add a new nest (peer) to the set of neighbours on behalf of 
  * a Nest client.
  */
  public void addClientNeighbour(anthill.NestId id) 
  {
   synchronized( neighbourManager )
   {
	neighbourManager.addNeighbour( id );
	saveNeighbourManager( neighbourManager );
   }    
  }

  /**
   * Add a new nest (peer) to the set of neighbours  on behalf of 
   * an Ant client. 
   */
  public void addAntNeighbour(anthill.NestId id) 
  {
   synchronized( neighbourManager )
   {
	neighbourManager.addNeighbour( id );
	saveNeighbourManager( neighbourManager );
   }
  }

  /**
   * Remove the entry associated with the provided NestId from the
   * set of known neighbours
   */
  public void removeNeighbour(anthill.NestId id) 
  {
   synchronized( neighbourManager )
   {
	neighbourManager.removeNeighbour( id );
	saveNeighbourManager( neighbourManager );
   }
  }

  /**
  * Return an array of known neighbours
  */
  public Neighbour[] getNeighbours() 
  {
   return neighbourManager.toArray();
  }

  /**
  * Passes an AntDescriptor to the Manager instance
  */
  public void deliver(AntDescriptor aDesc) throws RemoteException 
  {
   AntView aView = new AntView( this, aDesc, factory );
   manager.add( aView );
  }


  /**
  * Set the AnthillGroup instance. This method is invoked by the AnthillPeerWS
  * while performing its start up.
  */
  protected void setAnthillGroup(AnthillGroup grp) 
  {
   anthillGrp = grp;
  }

  /**
  * Set the nest id
  */
  protected void setId(NestId nestId)
  { 
   id = nestId;
  }

  /**
  * Set the PeerFinder instance
  */
  protected void setPeerFinder(PeerFinder finder) 
  {
   peerFinder = finder;
  }

  /**
  * Set the AntFactory instance to use
  */
  protected void setAntFactory(AntFactory aFactory) 
  {
   antFactory = aFactory;
  }

  /**
  * Set the Manager instance to use
  */
  protected void setManager(Manager manager) 
  {
   this.manager = manager;
  }
  
  /**
   * Return a reference to the Manager instance
   */
  public anthill.nest.Manager getManager()
  {
   return manager;
  }

  /**
   * Return a Trail object 
   */
  private Trail getTrail(boolean create)
  {
   if (trail == null && create) 
   {
	 trail = new Trail();
   }
   return trail;
  }
  
  /**
  * Set the NeighbourManager
  */
  protected void setNeighbourManager(NeighbourManager mng)
  {
	neighbourManager = mng;
  }
  
  /**
   * Return the Pheromone object indexed by the given key (antSpecies), which is
   * the species the ant invoking this method belongs to.
   */
  private Object getPheromone(String antSpecies)
  {
	antSpecies = antSpecies.trim().toLowerCase();
  
	synchronized( pheromoneHashTable )
	{
	  return pheromoneHashTable.get( antSpecies );
	}
  }
  
  /**
   * Add a new pheromone object in the hashtable, using as key value the species
   * the invoking ant belongs to.	 
   *
   * @param antSpecies The species of the invoking ant
   * @param ph The pheromone object 
   */
  private void setPheromone(String antSpecies, Object ph)
  {
	antSpecies = antSpecies.trim().toLowerCase();
  
	synchronized( pheromoneHashTable )
	{
	  pheromoneHashTable.put( antSpecies, ph );
	  savePheromone( pheromoneHashTable);
	}
  }
  
  /**
   * Updates the local pheromone object and stores it to disck in order to be able
   * to recover from crashes. If the pheromone instance did not exist, it would be
   * created using the <code>Object ph</code> parameter.
   *
   * @param antSpecies The species of the invoking ant
   * @param ph The pheromone object 
   */
  public void updatePheromone( String antSpecies, Object ph )
  {
	antSpecies = antSpecies.trim().toLowerCase();
  
	synchronized( pheromoneHashTable )
	{
	  if ( pheromoneHashTable.containsKey(antSpecies) )
	  {
	    pheromoneHashTable.remove( antSpecies );
	  }
	  pheromoneHashTable.put( antSpecies, ph );
	  savePheromone( pheromoneHashTable);
	}
  }
  
  /**
	 * Return the set of resources which satisfy the request, shaped as a Resource[].
	 * The <CODE>Object request</CODE> has to be an instance of the class
	 * <CODE>jxtaimpl.CmsStorage.Query</CODE>.
	 * The system will look for a Storage instance associated with the "species"
	 * parameter. If such a Storage didn't exist a StorageNotFoundException would be thrown.
	 *
	 * @param request The <code>jxtaimpl.CmsStorage.Query</code> instance
	 * @param species The ant species
	 */
  public Resource[] getResources(Object request, String species) throws StorageNotFoundException
  {
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = null;
	synchronized ( storage = (CmsStorage)DOCstorageHashTable.get(species) )
	{
	return storage.getResources( request );
	}
  }
  else
  {
	int size = factory.intValue(species);
	CmsStorage storage = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
	synchronized( storage )
	{
	DOCstorageHashTable.put(species, storage);
	return storage.getResources( request );
	}
  }
  }
  
  /**
	 * Return a Resource instance associated with the given key.
	 * The <CODE>Object key</CODE> has to be an instance of the class
	 * <CODE>jxtaimpl.CmsStorage.Query</CODE>.
	 * The system will look for a Storage instance associated with the "species"
	 * parameter. If such a Storage didn't exist a StorageNotFoundException would be thrown.
	 *
	 * @param request The <code>jxtaimpl.CmsStorage.Query</code> instance
	 * @param species The ant species
	 */
  public Resource getResource(Object request, String species)
  {
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = null;
	synchronized ( storage = (CmsStorage)DOCstorageHashTable.get(species) )
	{
	System.out.println("--- Nest: getResource()");
	return storage.getResource( request );
	}
  }
  else
  {
	int size = factory.intValue(species);
	CmsStorage storage = new CmsStorage(species, size, System.getProperty("user.dir"), downloadPipeId, downloadPipeName, downloadPipeType);
	synchronized( storage )
	{
	DOCstorageHashTable.put(species, storage);
	return storage.getResource( request );
	}
  }  
  }

  /**
	 *  Returns the <code>InputStream</code> for reading the resource locally stored
	 *  and identified by the given resource ID <code>resID</code>. The <code>spec</code>
	 *  parameter refers to the ant species the <code>Storage</code> is associated to.
	 *
	 * @param resID The byte array identifieing the resource whose <code>InputStream</code>
	 *        will be returned
	 * @param spec The byte array representing the ant species
	 */
  public InputStream getInputStreamForResource(byte[] resID, byte[] spec) throws StorageNotFoundException 
  {
	String species = new String( spec );
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = null;
	synchronized ( storage = (CmsStorage)DOCstorageHashTable.get(species) )
	{
	System.out.println("--- Nest: getInputStreamForResource()");
	InputStream is = storage.getInputStreamForContent( resID );
	return is;
	}
  }
  else
  {
	System.out.println("--- Nest: getInputStreamForResource() - Storage not found!!!");
	Throwable throwable = new Throwable("Nest.getInputStreamForResource() - Storage not found");
	throw new StorageNotFoundException( throwable );
  }  
  }
  
  /**
	 * Return the RequestSet instance
	 */
  public RequestSet getRequestSet()
  {
  return reqSet;
  }
  
  /**
	 * This method has to be invoked in order to unshare a resource
	 * locally stored.
	 *
	 * @param res The <code>jxtaimpl.storage.AnthillResource</code> representing the 
	 *        resource we want to unshare
	 */
  public void unShare(Resource res) throws StorageNotFoundException
  {
  AnthillResource antRes = (AnthillResource) res;
  String species = antRes.getSpecies().trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = null;
	synchronized ( storage = (CmsStorage)DOCstorageHashTable.get(species) )
	{
	storage.unShare( antRes );
	}
  }
  else
  {
	Throwable throwable = new Throwable("Storage not found");
	throw new StorageNotFoundException( throwable );  
  }
  }
  
  /**
	 * Add a resource to the Nest. The storage indexed by the key "species"
	 * will be used, and possibly created.
	 *
	 * @param resource The <code>AnthillResource</code> to add
	 * @param species The ant species
	 */
  public void addResource(Resource resource, String species) throws CapacityException
  {
  species = species.trim().toLowerCase();
  this.insert( resource, species ); 
  }
  
  /**
	 * Method invoked in order to send an Ant to another Nest.
	 * An IOException is thrown if the destination nest won't be reachable.
	 *
	 * @param id The <code>jxtaimpl.ids.NestId</code> ID of the destination nest
	 * @param ant the ant to be sent
	 * @param ttk The TTL value
	 */
  protected void move( NestId id, Ant ant, int ttl ) throws IOException     
  {
  try
  {
	Gate gate = ( Gate )anthillGrp.lookupService( ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID );
	gate.send(id, ant, ttl);
  }
  catch(net.jxta.exception.ServiceNotFoundException snfe)
  {
	System.out.println("ERROR - Nest: can't find GateService instance, try and restart the nest.");
  }
  }
  
  /**
	 * Method for cleaning up before being garbage collected
	 */
  public void finalize()
  {
  synchronized( neighbourManager )
  {
	saveNeighbourManager( neighbourManager );
  }
  synchronized( pheromoneHashTable )
  {
	savePheromone( pheromoneHashTable);
  }
  }
  
  /**
	 * Private method used for saving to disk the pheromone hash table 
	 */
  private void savePheromone( Hashtable pher )
  {
  try
  {
	FileOutputStream fos = new FileOutputStream( filePheromoneHashTable );
	ObjectOutputStream oos = new ObjectOutputStream( fos );
	oos.writeObject( pheromoneHashTable );
	fos.close();
  }
  catch(IOException ioe)
  {
	ioe.printStackTrace();
	System.out.println("--- Nest: could not save the pheromone hash table!");
  } 
  }
  
  /**
	 * Private method used for saving to disk the NeighbourManager instance
	 */
  private void saveNeighbourManager( NeighbourManager mng )
  {
  try
  {
	FileOutputStream fos = new FileOutputStream( fileNeighbourManager );
	ObjectOutputStream oos = new ObjectOutputStream( fos );
	oos.writeObject( mng );
	fos.close();
  }
  catch(IOException ioe)
  {
	ioe.printStackTrace();
	System.out.println("--- Nest: could not save the NeighbourManager instance!");
  }  
  }
  
  /**
	 *  Method for downloading a resource from its originating nest.
	 *
	 * @param res The <code>jxtaimpl.storage.AnthillResource<code> describing the resource
	 *        to be downloaded
	 */
  public void downloadResource( AnthillResource res ) throws CapacityException, 
								 StorageNotFoundException
  {
	String species = res.getSpecies();
  species = species.trim().toLowerCase();
  
  if ( DOCstorageHashTable.containsKey(species) )
  {
	CmsStorage storage = null;
	storage = (CmsStorage)DOCstorageHashTable.get(species);
	  Downloader downloader = new Downloader( res, anthillGrp, storage );
	downloader.start();
  }
  else
  {
	System.out.println("--- Nest: downloadResource() - Storage not found!!!");
	Throwable throwable = new Throwable("Storage not found");
	throw new StorageNotFoundException( throwable );
  }  
  }
  
  private static ModuleClassID ANTHILL_GROUP_GATE_SERVICE_MCLASS_ID;
  private static String CMS_DIR_NAME = "AnthillResources";
  private static String PHEROMONE_H_TAB_FILE_NAME = "pheromone.obj";
  private static String URL_H_TAB_FILE_NAME = "urls.obj";
  private static String NEIGHBOUR_MNG_FILE_NAME = "neighbours.obj";
  private static String SERIALIZATION = "serialization";
  private static String KNOWN_SPECIES = "species";
  //private static String CONF_DIR = "confdir";
  
  private File anthillDir;
  private File filePheromoneHashTable;
  private File fileURLHashTable;
  private File fileNeighbourManager;
  
  private int TTL;
  private String saveInfo;
  //private String confDir;
  private PeerFinder peerFinder;
  private AntFactory antFactory;
  private Manager manager;
  private RequestSet reqSet;
  private Trail trail;
  private NestId id;
  private AnthillGroup anthillGrp;
  private NeighbourManager neighbourManager;
  private Factory factory;
  private Hashtable pheromoneHashTable;
  private Hashtable URLHashTable;
  private Hashtable DOCstorageHashTable;
  
  private URL downloadPipeId;
  private String downloadPipeName;
  private String downloadPipeType;
}
