/******************************************************
 * File: MainToolbar.java
 * created 26-Jan-02 8:13:51 PM by frusso@cs.unibo.it
 */

package jxtaimpl.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.border.Border;
import jxtaimpl.Nest;
import jxtaimpl.ids.RequestId;
import jxtaimpl.storage.*;

public class MainToolbar extends JFrame
			 implements ActionListener
{
  private jxtaimpl.Nest localNest;
  private JButton buttonSearch;
  private JTextField keywordsField;
  private JLabel keywordsLabel;
  private JComboBox searchByBox;
  private JLabel searchByLabel;
  private JMenuBar menu;
  private JMenu fileMenu;
  private JMenu resourcesMenu;
  private JMenu infoMenu;
  private JMenu helpMenu;
  private JMenuItem fileMenuExit;
  private JMenuItem resourcesMenuShare;
  private JMenuItem resourcesMenuUnshare;
  private JMenuItem resourcesMenuLocalStorage;
  private JMenuItem infoMenuInfo;
  private JMenuItem helpMenuHelp;
  private JPanel panel;
  
  private boolean RESIZABLE = false;

  private String TITLE = "Anthill&Jxta: Main Toolbar";
  private String BUTTON_SEARCH = "Search!";

  
  /**
  * Constructor of the main window. That's a toolbar which allows a user to 
  * interact with the local nest. Its functionalities are: searching for new
  * resources, sharing local resources within the Anthill network, unshare
  * them, explore the local storage.
  */
  public MainToolbar(Nest nest)
  {
	localNest = nest;
	setTitle( TITLE );
	setSize( 700, 120 );
	setResizable( RESIZABLE );
	addWindowListener( new WindowAdapter()
					  {
				public void windowClosing( WindowEvent e)
				{
				  int res = JOptionPane.showConfirmDialog( null, "Are you sure you wish to exit?", "Anthill&Jxta: Exit?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				if ( res == JOptionPane.YES_OPTION )
				{
				  localNest.finalize();
				  System.exit(0);
				} 
				}
			 }
			);
	buttonSearch = new JButton(BUTTON_SEARCH);
	buttonSearch.addActionListener( this );
	buttonSearch.setEnabled( true );
	keywordsField = new JTextField();
	keywordsField.addActionListener( this );
	keywordsLabel = new JLabel("Keywords:");
	searchByBox = new JComboBox();
	searchByBox.addItem("Name");
	searchByBox.addItem("Type");
	searchByBox.addItem("Description");
	searchByBox.addActionListener( this );
	searchByLabel = new JLabel("Search by:");
	menu = new JMenuBar();
	fileMenu = new JMenu("File");
	fileMenuExit = new JMenuItem("Exit");
	fileMenuExit.addActionListener( this );
	fileMenu.add(fileMenuExit);
	fileMenu.setMnemonic('F');
	resourcesMenu = new JMenu("Resources");
	resourcesMenuShare = new JMenuItem("Share");
	resourcesMenuShare.addActionListener( this );
	resourcesMenuUnshare = new JMenuItem("Unshare");
	resourcesMenuUnshare.addActionListener( this );
	resourcesMenuLocalStorage = new JMenuItem("Local Storage");
	resourcesMenuLocalStorage.addActionListener( this );
	resourcesMenu.add(resourcesMenuShare);
	resourcesMenu.add(resourcesMenuUnshare);
	resourcesMenu.add(resourcesMenuLocalStorage);
	resourcesMenu.setMnemonic('R');
	infoMenu = new JMenu("Info");
	infoMenuInfo = new JMenuItem("Product info");
	infoMenuInfo.addActionListener( this );
	infoMenu.add(infoMenuInfo);
	infoMenu.setMnemonic('I');
	helpMenu = new JMenu("Help");
	helpMenuHelp = new JMenuItem("User guide");
	helpMenuHelp.addActionListener( this );
	helpMenu.add(helpMenuHelp);
	helpMenu.setMnemonic('H');
	menu.add(fileMenu);
	menu.add(resourcesMenu);
	menu.add(infoMenu);
	menu.add(helpMenu);
	
	panel = new JPanel();
	GridBagLayout lay = new GridBagLayout();
	Border etched = BorderFactory.createEtchedBorder();
	panel.setLayout(lay);
	panel.setBorder(etched);

	panel.add(searchByBox, createConstraints(0,1,1,1,1,1));
	panel.add(keywordsLabel, createConstraints(1,0,1,1,0,0));
	panel.add(keywordsField, createConstraints(1,1,1,1,10,10));
	panel.add(buttonSearch, createConstraints(2,1,1,1,1,1));
	
	getContentPane().add(menu, "North");
	getContentPane().add(panel, "Center");
  }

  public void actionPerformed ( ActionEvent evt )
	{ 
	Object src = evt.getSource();
	
	if (src == buttonSearch)
	{
	  String keywords = null;
	  keywords = keywordsField.getText().trim();
	  
	  if ( keywords.length()==0 )
	  {
		JOptionPane.showMessageDialog( this, "Type the keyword(s) first, please", "Anthill&Jxta: Keywords", JOptionPane.ERROR_MESSAGE);
	  }
	  else
	  {
		RequestId reqId = new RequestId( localNest.getNestId() );
		Query query = new Query( (String)searchByBox.getSelectedItem(), keywords );
		QueryResults results = new QueryResults( localNest, searchByBox.getSelectedItem().toString().trim(), keywords );
		localNest.request( reqId, query, results);
		results.show();
	  }
	}
	else
	{
	  if (src == fileMenuExit)
	  {
		int res = JOptionPane.showConfirmDialog( this, "Are you sure you wish to exit?", "Anthill&Jxta: Exit?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
		if ( res == JOptionPane.YES_OPTION )
		{
		  localNest.finalize();
		  System.exit(0);
		}
	  }
	  else
	  {
		if (src == resourcesMenuShare)
		{
		  JFileChooser fileChooser = new JFileChooser();
		  fileChooser.setCurrentDirectory( new File(System.getProperty("user.home")) );
		  fileChooser.setMultiSelectionEnabled( true );
		  fileChooser.showDialog( this, "Share");
		  
		  File[] files = fileChooser.getSelectedFiles();
		  
		  for(int i=0; i< files.length; i++)
		  {
			SharingForm form = new SharingForm( localNest, files[i] );
			form.show();
		  }
		}
		else
		{
		  if (src == resourcesMenuUnshare)
		  {
			  anthill.Resource[] res = localNest.getAllResources();
			  if( res!=null )
			  {
				AnthillResource[] antRes = new AnthillResource[ res.length ];
				for(int i=0; i<res.length; i++)
				{
				  antRes[i] = (AnthillResource)res[i];
				}
				UnsharingForm form = new UnsharingForm( localNest, antRes );
				form.show();
			  }
			  else
			  {
				JOptionPane.showMessageDialog( this, "The local storage is empty.", "Anthill&Jxta: Local Storage", JOptionPane.INFORMATION_MESSAGE);
			  }
		  }
		  else
		  {
			if (src == resourcesMenuLocalStorage)
			{
			  anthill.Resource[] res = localNest.getAllResources();
			  if( res!=null )
			  {
				AnthillResource[] antRes = new AnthillResource[ res.length ];
				for(int i=0; i<res.length; i++)
				{
				  antRes[i] = (AnthillResource)res[i];
				}
				LocalStorageView localView = new LocalStorageView( antRes );
				localView.show();
			  }
			  else
			  {
				JOptionPane.showMessageDialog( this, "The local storage is empty.", "Anthill&Jxta: Local Storage", JOptionPane.INFORMATION_MESSAGE);
			  }
			}
			else
			{
			  if (src == infoMenuInfo)
			  {
			  }
			  else
			  {
				if (src == helpMenuHelp)
				{
				}
			  }
			}
		  }
		}
	  }
	}
  }
  
  private GridBagConstraints createConstraints(int gx, int gy, int gw, int gh, int wx, int wy)
	{
	GridBagConstraints c = new GridBagConstraints();

	c.gridx = c.RELATIVE;
	c.gridy = c.RELATIVE;
	c.gridwidth = gw;
	c.gridheight = gh;
	c.weightx = wx;
	c.weighty = wy;
	 c.insets = new Insets(1,1,1,1);
	c.fill = GridBagConstraints.HORIZONTAL;
  
	return c;
	}
  
}
