/* Generated by Together */

package jxtaimpl.storage;

import java.io.*;
import java.lang.reflect.*;
import java.net.URL;
import java.text.BreakIterator;
import java.util.*;

import net.jxta.share.*;

import anthill.*;
import anthill.storage.*;

/**
 * This class is in charge of managing the resources to be shared within the Anthill network.
 * @author Francesco Russo (frusso@cs.unibo.it) 
 */
public class CmsStorage implements QueryStorage, ResourceManager 
{
  /**
  * This is the constructor. The "species" parameter is the invoking ant's species.
  * Once invoked it will create in the user's home directory
  * a new folder called AnthillResources, which will be used for storing information
  * about the resources to be shared with all the other nests.
  *
  * @param species The ant species this storage is associated to
  * @param size The desired storage's size expressed in bytes
  * @param conDir The folder in which all the information will be stored
  * @param pipeId This is the JXTA pipeID of the local <code>jxtaimpl.DownloadService</code>
  *               input-pipe waiting for download requests to come
  * @param pipeName The pipe's logical name
  * @param pipeType The pipe's type (point-to-point, propagate, secure)
  */
  public CmsStorage(String species, long size, String confDir, URL pipeId, String pipeName, String pipeType) 
  {
	boolean found = false;
	boolean share = false;
  
	this.species = species;
  
  	this.pipeId = pipeId;
	this.pipeName = pipeName;
	this.pipeType = pipeType;
	
	//System.out.println("--- CmsStorage: "+pipeId.toString()+" "+pipeName+" "+pipeType);
  
	//* tests wheter the main directory exists or not
	if ( anthillDir == null )
	{
	  //anthillDir = new File(System.getProperty("user.home")+File.separator+CMS_DIR_NAME);
	  anthillDir = new File( confDir+File.separator+CMS_DIR_NAME );
	  if ( !anthillDir.exists() )
	  {
		anthillDir.mkdir();
		resFolder = new File(confDir+File.separator+CMS_DIR_NAME+File.separator+DOC_DIR_NAME);
		resFolder.mkdir();
	  }
	  else
	  {  //* tests whether the "sharedResource" folder exists or not 
		resFolder = new File(confDir+File.separator+CMS_DIR_NAME+File.separator+DOC_DIR_NAME);
		if ( !resFolder.exists() )
		{
		  resFolder.mkdir();
		}
	  }
	}
	
	//* tests whether resFolder contains a sub-folder for "species"
	String[] resFolderChildren = resFolder.list();
	if ( resFolderChildren == null )
	 {
	  //* there are no sub-folders, so let's create the one we need
	  speciesDir = new File( resFolder, species );
	  speciesDir.mkdir();
	}
	else
	{
	  //* there are some sub-folders
	  for (int i = 0; i < resFolderChildren.length; i++)
	  {
		if ( resFolderChildren[i].trim().toLowerCase().equals(species.trim().toLowerCase()) )
		{
		  //* we have found our folder
		  speciesDir = new File( resFolder, species );
		  found = true;
		  break;
		}
	  }
	  if ( !found )
	  {
		//* we didn't find the folder we are interested in
		//* let's create it
		speciesDir = new File( resFolder, species );
		speciesDir.mkdir();
	  }
	}
	
	found = false;
	String[] speciesDirChildren = speciesDir.list();
	if ( speciesDirChildren == null )
	{
	  //* no sub-folders
	  //* let's create the respurces repository
	  speciesResDir = new File( speciesDir, RES_DIR_NAME );
	  speciesResDir.mkdir();
	}
	else
	{
	  //* there are some sub-folders
	  for (int i = 0; i < speciesDirChildren.length; i++)
	  {
		if ( speciesDirChildren[i].trim().toLowerCase().equals(RES_DIR_NAME.trim().toLowerCase()) )
		{
		  //* we have found our folder
		  speciesResDir = new File( speciesDir, RES_DIR_NAME );
		  //* the folder already exists
		  found = true;
		  break;
		}
	  } 
	  if ( !found )
	  {
		//* we didn't find the folder we are interested in
		//* let's create it
		speciesResDir = new File( speciesDir, RES_DIR_NAME );
		speciesResDir.mkdir();
	  }
	}
	//* now we have to instantiate the contentManager 
	try
	{
	  contentManager = new ContentManagerImpl( speciesDir, true);
	}
	catch(IOException ioe)
	{
	  ioe.printStackTrace();
	  System.out.println("--- CmsStorage: could not instantiate the ContentManager, giving up!");
	  System.exit(1);
	}
	diskSpace = size;
	leftDiskSpace = diskSpace - this.computeSize();
  }
  
  
  /**
  * This private method computes the disk space used for storing the shared
  * resources.
  */
  private long computeSize()
  {
	long tot = 0;
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i < contents.length; i++)
	{
	  long dim = contents[i].getContentAdvertisement().getLength();
	  if ( dim != -1 )
	  {
		tot = tot + dim;
	  }
	  else
	  {
		System.out.println("WARNING --- CmsStorage: resource "+contents[i].getContentAdvertisement().getName()+" states a -1 KB dimension!"); 
	  }
	}
	return tot;
  }
  
  
  /**
  * This method does nothing, it has no sense in the <CODE>jxtaimpl</CODE>
  * implementation.
  */
  public void clear()
  {
  }
  
  /**
	*  Returns the first resource which satisfies the query (key). If the key
	*  is not associated with any resource, a null value is returned.
	* In the <CODE>jxtaimpl</CODE> implementation of Anthill, the key object has
	* to be an instance of the the class Query belonging to this same package.
	*
	*  @param key  whose associated resource to retreive.
	*  @return resource with which the specified key is associated.
	*/
  public Resource getResource(Object key)
  {
	System.out.println("--- CmsStorage: getResource()");
	
	Query query = (Query) key;
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i<contents.length; i++)
	 {
	  String value = this.getAttribute(contents[i], query.getElementName());
	  
	  if ( (value!=null) && this.satisfyQuery(value, query.getValues()) )
	  {
		//* we have found a content which satisfies the query
		//* I've to instantiate an AnthillResource and return it
		System.out.println("--- CmsStorage: getResource() FOUND!");
		return new AnthillResource( contents[i], species, pipeId, pipeName, pipeType );
	  }
	}
	System.out.println("--- CmsStorage: getResource() NOT FOUND!");
	//* nothing has been found
	return null;      
  }
  
  /**
  *  Returns the resources associated with the specified key. If the key
  *  is not associated with any resource, a null value is returned.
  *  In the <CODE>jxtaimpl</CODE> implementation of Anthill, the key object has
  *  to be an instance of the the class Query belonging to this same package.
  *
  *  @param key  whose associated resource to retreive.
  *  @return resource with which the specified key is associated.
  */
  public Resource[] getResources(Object key)
  {
	System.out.println("--- CmsStorage: getResources()");
  
	Vector res = new Vector();
	Query query = (Query) key;
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i<contents.length; i++)
	 {
	  String value = this.getAttribute(contents[i], query.getElementName());
	  
	  if ( (value!=null) && this.satisfyQuery(value, query.getValues()) )
	  {
		//* we have found a content which satisfies the query
		res.addElement( new AnthillResource(contents[i], species, pipeId, pipeName, pipeType) );
	  }
	}
	if ( res.isEmpty() )
	{
	  System.out.println("--- CmsStorage: getResources() - NOT FOUND! ");
	  return null;
	}     
	  
	Resource[] result = new Resource[res.size()];
	for (int i=0; i<res.size(); i++)
	{
	  result[i] = (Resource) res.elementAt(i);  
	}
	System.out.println("--- CmsStorage: getResources() - FOUND SOME RESOURCES! ");
	return result;
  }
  
  /**
  * This method will return the complete set of resources stored in the
  * local nest if the storage is not empty, else it returns <CODE>null</CODE>.
  */
  public Resource[] getAllResources()
  {
	Content[] contents = contentManager.getContent();
	
	if ( contents.length > 0 )
	{
	  Resource[] res = new Resource[ contents.length ];
	  for (int i=0; i<contents.length; i++)
	  {   
		res[ i ] = (Resource) new AnthillResource( contents[i], species, pipeId, pipeName, pipeType );
	  }
	  return res;
	}   
	return null;
  }

  /**
  * Returns true if one resource satisfing the query <CODE>key</CODE>,
  * (please see <CODE>Query</CODE> javadocs) is contained in
  * the storage, false otherwise. The <CODE>key</CODE> is meant to be a couple
  * like this: <CODE>(elementName, values)</CODE>. Both of them are meant to be
  * <CODE>String</CODE>. All the interesting values contained in <CODE>values</CODE>
  * must be separated by a blank space. It doesn't matter whether they are upper-case
  * or not.
  *
  *  @param key  the key to be searched
  *  @return true if the resource associated to <CODE>key</CODE> is contained in
  *    the storage
  */
  public boolean containsResource(Object key)
  {
	Query query = (Query) key;
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i<contents.length; i++)
	{
	  String value = this.getAttribute(contents[i], query.getElementName());
	  
	  if ( (value!=null) && this.satisfyQuery(value, query.getValues()) )
	  {
		//* we have found a content which satisfies the query
		return true;
	  }
	}
	
	//* nothing has been found
	return false;
  }
  
  /**
  * This private method, given a Content and an element name, searches for getter
  * methods associated with that element name and if one is found returns its value,
  * otherwise <CODE>null</CODE> will be returned.
  */
  private String getAttribute(Content content, String name)
  {
	try
	{
	  ContentAdvertisement adv = content.getContentAdvertisement();
	  Class contAdvClass = Class.forName("net.jxta.share.ContentAdvertisement");
	  Method[] methods = contAdvClass.getMethods();
	  int i = 0;
	
	  for (i = 0; i < methods.length; i++)
	  {
		if ( methods[i].getName().startsWith("get") )
		{
		  if ( name.trim().toLowerCase().equals( methods[i].getName().substring(3).trim().toLowerCase() ) )
		  {
			Object obj = methods[i].invoke( adv, null );
			return obj.toString();
		  }
		}
	  }
	}
	catch(ClassNotFoundException cnfe)
	{
	  cnfe.printStackTrace();
	}
	catch(IllegalAccessException iae)
	{
	  iae.printStackTrace();
	}
	catch(InvocationTargetException ite)
	{
	  ite.printStackTrace();
	}
	return null;
  }
  
  /**
  * This private method checks whether or not the key value <CODE>cond</CODE>
  * can be found within <CODE>String str</CODE>.
  * The key value might be one <CODE>String</CODE> or a sequence of
  * <CODE>String</CODE> values separated by blank space(s).
  * This method will test whether a sub-string of <CODE>cond</CODE> can be found
  * within <CODE>String str<CODE> or not. The sub-strings are tested sequentially,
  * and the method will return once the first matching occours.
  */
  private boolean satisfyQuery( String str, String cond )
  {
	BreakIterator iterator = BreakIterator.getWordInstance();
	StringTokenizer condTokenizer = new StringTokenizer( cond );
	
	while( condTokenizer.hasMoreElements() ) 
	{
	  String tmpCond = condTokenizer.nextToken();
  
	  int i = 0;
	  int j = 0;
	  iterator.setText( str );
	  
	  for ( j = iterator.next(); j != BreakIterator.DONE; i = j, j = iterator.next() ) 
	  {
		String tmp = str.substring(i,j);
		if ( tmp.trim().toLowerCase().equals(tmpCond.trim().toLowerCase()) )
		  return true;
	  }
	}
	
	return false;
  }
  
	/**
	*  Adds a resource to the storage and associates it to the specified
	*  key if and only if the same resource is not already locally stored. 
	*  If the remaining capacity of the storage is not sufficient to
	*  store the resource, a <CODE>CapacityException</CODE> is thrown
	*  containing information about the remaining capacity. Method
	*  <CODE>getContentLength()</CODE> may be invoked on the resource
	*  to determine the amount of space required by the resource. 
	*  In order to be stored on persistent storage, resource must 
	*  implement the <CODE>Serializable</CODE> interface.
	*
	*  @deprecated The <code> public void addResource(Object key, Resource resource, byte[] buffer)
	*			   </code> method should be used in place of this one	
	*  @param key   the key identifying the resource
	*  @param resource  the resource to be added
	*  @throws CapacityException  raised if the given resource object
	*    exceeds the total available capacity for this storage.
	*/
  public void addResource(Object key, Resource resource) throws CapacityException
  {
	if( !alreadyStored(resource) )
	{
	  if ( (resource.size()>0) && ((leftDiskSpace-resource.size())>0) )
	  {
		try
		{
		  //* firstly let's copy the resource locally, and then let's share it
		  InputStream is = resource.getInputStream();
		  int BYTES = is.available();
		  byte[] buffer = new byte[BYTES];
		  String fileName = ((AnthillResource)resource).getResourceName();
		  File outFile = new File( speciesResDir, fileName );
		  FileOutputStream fos = new FileOutputStream( outFile );
		  
		  while( is.read( buffer ) != -1  )
		  {
			fos.write( buffer );
		  }
		  
		  is.close();
		  fos.close();
		  
		  contentManager.share( outFile, fileName, resource.getAttribute("type"), resource.getAttribute("description") );
		  leftDiskSpace = leftDiskSpace - resource.size();
		}
		catch(IOException ioe)
		{
		  ioe.printStackTrace();
		}
	  }
	  else
	  {  //* not enough disk space left
		throw new CapacityException();
	  }
	}
  }
  
  /**
  *  Adds the resource described by <code>Resource resource</code> and contained in
  *  <code>byte[] buffer</code>. This method should be invoked in place of the 
  *  <code>addResource(Object key, Resource resource)</code> method for locally adding
  *  a new resource. 
  *
  *  @param key The key identifieing the resource. This might be <code>null</code> since 
  *             in this method's signature there is a resource descriptor as well
  *  @param resource This is the resource descriptor which has to be an <code>AnthillResource</code>
  *                  instance
  *  @param buffer This is the content which has to be locally stored
  *  @throws CapacityException
  */
  public void addResource(Object key, Resource resource, byte[] buffer) throws CapacityException
  {
	if( !alreadyStored(resource) )
	{
	  if ( (resource.size()>0) && ((leftDiskSpace-resource.size())>0) )
	  {
		try
		{
		  //* firstly let's copy the resource locally, and then let's share it
		  String fileName = ((AnthillResource)resource).getResourceName();
		  File outFile = new File( speciesResDir, fileName );
		  FileOutputStream fos = new FileOutputStream( outFile );
		  fos.write( buffer );
		  fos.close();
		  
		  contentManager.share( outFile, fileName, resource.getAttribute("type"), resource.getAttribute("description") );
		  leftDiskSpace = leftDiskSpace - resource.size();
		}
		catch(IOException ioe)
		{
		  ioe.printStackTrace();
		}
	  }
	  else
	  {  //* not enough disk space left
		throw new CapacityException();
	  }
	}
  }
  
  /**
  * This private method tests wether the <CODE>Content</CODE> associated with
  * the given <CODE>AnthillResource</CODE> already exists or not in the local
  * storage.
  *
  */
  private boolean alreadyStored(Resource res)
  {
	Content cont = ((AnthillResource)res).getContent();
	ContentAdvertisement cAdv = cont.getContentAdvertisement();
	
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i<contents.length; i++)
	{
	  ContentAdvertisement tmpAdv = contents[i].getContentAdvertisement();
	  if ( tmpAdv.equals(cAdv) )
		return true;
	}
	
	return false;
  }

  /**
  * This private method tests wether a resource exposing the 
  * same description of the input parameter already exists 
  * or not in the local storage.
  *
  */
  private boolean alreadyStored(String description)
  {
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i<contents.length; i++)
	{
	  ContentAdvertisement tmpAdv = contents[i].getContentAdvertisement();
	  String desc = tmpAdv.getDescription();
	  if ( desc.equals(description) )
		return true;
	}
	
	return false;
  }

  /**
  * This method removes the first resource which satisfies the given query.
  *
  * @param query This object must be a <code>jxtaimpl.storage.Query</code> instance or
  *              has to extend it for this method to work. It contains the couple <code>
  *              (ElementName, Value)</code> identifieing the resource we want to remove
  */
  public void removeResource(Object query)
  {
	Query _query = (Query) query;
	Content[] contents = contentManager.getContent();
	
	for (int i=0; i<contents.length; i++)
	 {
	  String value = this.getAttribute(contents[i], _query.getElementName());
	  
	  if ( (value!=null) && this.satisfyQuery(value, _query.getValues()) )
	  {
		//* this resource (a Content) has to be removed from the
		//* file system just if it resides in the path
		//* user's_home/AnthillResources/sharedResources/species/resources/
		
		String fileName = contents[i].getContentAdvertisement().getName();
		File victimFile = new File( speciesResDir, fileName );
		if ( victimFile.exists() )
		{
		  victimFile.delete();
		}
		victimFile = null;
		
		long dim = contents[i].getContentAdvertisement().getLength();
		if ((dim>0) && ((leftDiskSpace - dim)>0))
		  leftDiskSpace = leftDiskSpace - dim; 
		
		//* now it has to be removed from the ContentManager
		try
		{
		  contentManager.unshare( contents[i] );
		}
		catch(IOException ioe)
		{
		  ioe.printStackTrace();
		}
	  }
	} 
  }
  
  /**
  * This method is used by the Nest instance in order to share new resources 
  * not found by ants, but passed by the application level. With this method
  * the resource will not be copied to the folder in charge of storing all the
  * contents to be shared.
  *
  * @param file The file representing the resource we want to share
  * @param name The logical name for the resource
  * @param type The resource's type
  * @param description A logical description of the resource
  */
  public void share( File file, String name, String type, String description ) throws ResourceAlreadyStoredException,
  																					  CapacityException
  {
	try
	{
	  if( !alreadyStored(description) )
	  {
	  	if( (file.length()>0) && ((leftDiskSpace-file.length())>0) )
		{
	  		contentManager.share( file, name, type, description );
		}
		else
		{
			throw new CapacityException();
		}
	  }
	  else
	  {
	  	throw new ResourceAlreadyStoredException();
	  }
	}
	catch(IOException ioe)
	{
	  ioe.printStackTrace();
	  System.out.println("--- CmsStorage: could not share resource "+file.getName());
	}
  }
  
  /**
  * This method is used by the Nest instance in order to share new resources 
  * not found by ants, but passed by the application level. With this method
  * the resource will not be copied to the folder in charge of storing all the
  * contents to be shared.
  *
  * @param file The <code>byte</code> array representing the resource we want to share
  * @param name The logical name for the resource
  * @param type The resource's type
  * @param description A logical description of the resource
  */
  public void share( byte[] content, String name, String type, String description ) throws ResourceAlreadyStoredException,
  																						   CapacityException
  {
	try
	{
	  if( !alreadyStored(description) )
	  {
	  	if( (content.length>0) && ((leftDiskSpace-content.length)>0) )
		{
	  		//* firstly let's copy the resource locally, and then let's share it
			File outFile = new File( speciesResDir, name );
			FileOutputStream fos = new FileOutputStream( outFile );
			fos.write( content );
			fos.close();
	    
			contentManager.share( outFile, name, type, description );
		}
		else
		{
			throw new CapacityException();
		}
	  }
	  else
	  {
	  	throw new ResourceAlreadyStoredException();
	  }
	}
	catch(IOException ioe)
	{
	  ioe.printStackTrace();
	  System.out.println("--- CmsStorage: could not share resource "+name);
	}
  }
  
  /**
  * This method removes the specified resouce from the local storage.
  *
  * @param The <code>AnthillResource</code> instance we want to unshare
  */
  public void unShare( AnthillResource resource ) 
  {
	//* this resource (a Content) has to be removed from the
	//* file system just if it resides in the path
	//* user's_home/AnthillResources/sharedResources/species/resources/
		
	Content cont = resource.getContent();
	
	String fileName = resource.getResourceName();
	File victimFile = new File( speciesResDir, fileName );
	if ( victimFile.exists() )
	{
	  victimFile.delete();
	}
	victimFile = null;
		
	long dim = cont.getContentAdvertisement().getLength();
	if ((dim>0) && ((leftDiskSpace - dim)>0))
	  leftDiskSpace = leftDiskSpace - dim; 
	  
	//* now it has to be removed from the ContentManager
	try
	{
	  contentManager.unshare( cont );
	}
	catch(IOException ioe)
	{
	  ioe.printStackTrace();
	}
  }
  
  /**
  *  Returns the complete set of resources stored in this storage object
  *  using an iterator.
  *
  *  @return iterator representing this storage object, allowing to
  *    inspect its content.
  */
  public Iterator iterator()
  {
	Content[] cont = contentManager.getContent();
	Vector vect = new Vector();
	
	for (int i=0; i<cont.length; i++)
	{
	  vect.addElement( cont[i] ); 
	}
	
	return vect.iterator();
  }
  
  /**
  *  Returns the initial capacity of this generic storage.
  */
  public long getCapacity()
  {
	return diskSpace;
  }
  
  /**
  *  Returns the remaining capacity of this generic storage.
  */
  public long getRemainingCapacity()
  {
	return leftDiskSpace;
  }
  
  /**
  *  Gets the <code>InputStream</code> for reading in the resource identified by the
  *  the given contentId. The <code>byte</code> array is actually the <code>String</code>
  *  rapresentation of the <code>ContentIdImpl</code> instance.
  *  This method should only be used by the <code>Nest</code> instance in order to
  *  upload the resource identified by the given contentID.
  *
  *  @param contentId The byte array representing the <code>String</code> from which
  *                   the <code>ContentIdImpl</code> identifieing the resource
  *                   we are interested in should be generated
  */
  public InputStream getInputStreamForContent( byte[] contentId )
  {
  	ContentIdImpl contentID = new ContentIdImpl( new String(contentId) ); 
  	Content[] contents = contentManager.getContent( contentID );
	
	if( contents.length > 0 )
	{
		try
		{
			return contents[0].getInputStream();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}
		return null;
	}
	else
	{
		return null;
	}
  }
  
  
  private ContentManagerImpl contentManager;
  private long diskSpace;
  private long leftDiskSpace;
  private File speciesDir;
  private File speciesResDir;
  private String species;
  
  private URL pipeId;
  private String pipeName;
  private String pipeType;
  
  private static File anthillDir = null;
  private static File resFolder = null;
  private static String CMS_DIR_NAME = "AnthillResources";
  private static String DOC_DIR_NAME = "sharedResources";
  private static String RES_DIR_NAME = "resources";
  private static String CMS_FILE_NAME = "shares.ser";
}
