/******************************************************
 * File: AntDialog.java
 * created 15-May-02 2:55:53 PM by root
 */


package techrep;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import jxtaimpl.Nest;

import org.w3c.dom.Element;
import techrep.TrBibReader.BibFormatException;

public class AntDialog extends JDialog implements ActionListener
{
  public AntDialog(JFrame frame, Nest nest)
  {
	super(frame, "Sharing AntSpecies", true);
  	this.nest = nest;
	Container contentPane = getContentPane();
	JPanel p1 = new JPanel();
	p1.setLayout( new GridLayout(2,2) );
	p1.add( new JLabel("AntSpecies") );
	p1.add( name = new JTextField() );
	contentPane.add("Center", p1);
	
	JPanel p2 = new JPanel();
	okButton = addButton( p2, "OK" );
	cancelButton = addButton( p2, "Cancel" );
	contentPane.add("South", p2);
	setSize(270, 110);
  }
  
  JButton addButton(Container c, String name)
  {
  	JButton button = new JButton(name);
	button.addActionListener( this );
	c.add(button);
	return button;
  }
  
  public void actionPerformed(ActionEvent evt)
  {
  	Object src = evt.getSource();
	
	if (src == okButton)
	{
			String species = name.getText().trim();
			if ( species.length()==0 )
	  		{
				JOptionPane.showMessageDialog( this, "Fill in the form, please", "Anthill&Jxta: Missing data", JOptionPane.ERROR_MESSAGE);
				return;
	  		}
			
	  		JFileChooser fileChooser = new JFileChooser();
		    fileChooser.setCurrentDirectory( new File(System.getProperty("user.home")) );
		 	fileChooser.setMultiSelectionEnabled( false );
		  	fileChooser.showDialog( this, "Share");
			File bibFile = fileChooser.getSelectedFile();
		    techrep.TrBibReader.Reader bibReader = null;
		  	try
			{
				bibReader = new techrep.TrBibReader.Reader( bibFile );
		    	bibReader.parse();
			}
			catch(FileNotFoundException fnfe)
			{
				fnfe.printStackTrace();
				JOptionPane.showMessageDialog( this, "File not found", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
				this.dispose();
			}
			catch(BibFormatException bfe)
			{
				bfe.printStackTrace();
				JOptionPane.showMessageDialog( this, "File format exception", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
				this.dispose();
			}
			catch(ParserConfigurationException pce)
			{
				pce.printStackTrace();
				JOptionPane.showMessageDialog( this, "Parser exception", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
				this.dispose();
			}
			catch(TransformerException te)
			{
				te.printStackTrace();
				JOptionPane.showMessageDialog( this, "Transformer exception", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
				this.dispose();
			}
			
			Vector domDocs = bibReader.getDocuments();
			Vector strDocs = bibReader.getDocumentsAsStrings();
			while( !domDocs.isEmpty() )
			{
				org.w3c.dom.Document dom = (org.w3c.dom.Document) domDocs.remove(0);
				Element root = dom.getDocumentElement();
				String resLocation = root.getAttribute(RES_LOCATION);
				resLocation = resLocation.trim().substring(4).trim();
				String fileName = extractFileName( resLocation );
				String descr = (String) strDocs.remove(0);
		  		
				ContentUploader thread = new ContentUploader( fileName, resLocation, descr, species, nest );
				thread.start();
			}
			/*
			JOptionPane.showMessageDialog( this, "Done!", "Anthill&Jxta: Shared!", JOptionPane.INFORMATION_MESSAGE);
			this.dispose();
			*/
	}
	else
	{
	  if (src == cancelButton)
	  	this.dispose();
	}
  }
  
  /**
  *  This method extracts the file name from the given URL
  */
  private String extractFileName(String url)
  {
  	StringTokenizer tok = new StringTokenizer( url, "/" );
	String fileName = null;
	
	while( tok.hasMoreTokens() )
	{
		fileName = tok.nextToken().trim();
	}
	
	return fileName;
  }
  
  private JButton okButton;
  private JButton cancelButton;
  private JTextField name;
  private Nest nest;
  
  private String RES_LOCATION = "OTHER_ACCESS";
}
