/******************************************************
 * File: SharingForm.java
 * created 27-Jan-02 3:35:11 PM by root
 */

package techrep;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.border.Border;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import jxtaimpl.*;
import jxtaimpl.storage.ResourceAlreadyStoredException;

import techrep.TrBibReader.Bib2XML;

import anthill.storage.CapacityException;

public class SharingForm extends JFrame
			 implements ActionListener
{
  private JLabel resourceNameLabel;
  private JLabel resourceIDLabel;
  private JLabel resourceEntryLabel;
  private JLabel resourceOrganizationLabel;
  private JLabel resourceTitleLabel;
  private JLabel resourceDateLabel;
  private JLabel resourceCopyrightLabel;
  private JLabel resourceOtherAccessLabel;
  
  private JTextField resourceIDText;
  private JTextField resourceEntryText;
  private JTextField resourceOrganizationText;
  private JTextField resourceTitleText;
  private JTextField resourceDateText;
  private JTextField resourceCopyrightText;
  private JTextField resourceOtherAccessText;
  
  private JLabel antSpeciesLabel;
  private JTextField antSpeciesText;

  private JButton cancelButton;
  private JButton nextButton;
  
  private String ID;
  private String entry;
  private String organization;
  private String title;
  private String date;
  private String copyright;
  private String otherAccess;
  private String species;
  
  private File file;
  private Nest nest;
  
  public SharingForm(Nest localNest, File file) 
  {
	this.file = file;
	nest = localNest;
  
	setTitle("Anthill&Jxta: Resource Form");
	setSize(450,500);
	setResizable(true);
	addWindowListener( new WindowAdapter()
					  {
						public void windowClosing( WindowEvent e)
						{
						  dispose();
						}
			 		  }
					 );
	resourceNameLabel = new JLabel("Name: "+file.getName());
	resourceIDLabel = new JLabel("ID: ");
	resourceEntryLabel = new JLabel("Entry: ");
	resourceOrganizationLabel = new JLabel("Organization: ");
	resourceTitleLabel = new JLabel("Title: ");
	resourceDateLabel = new JLabel("Date: ");
	resourceCopyrightLabel = new JLabel("Copyright: ");
	resourceOtherAccessLabel = new JLabel("Access: ");
	
	antSpeciesLabel = new JLabel("Ant Species: ");
	
	resourceIDText = new JTextField();
	resourceEntryText = new JTextField();
	resourceOrganizationText = new JTextField();
	resourceTitleText = new JTextField();
	resourceDateText = new JTextField();
	resourceCopyrightText = new JTextField();
	resourceOtherAccessText = new JTextField();
	
	antSpeciesText = new JTextField();
	
	cancelButton = new JButton("Cancel");
	cancelButton.addActionListener( this );
	nextButton = new JButton("Next");
	nextButton.addActionListener( this );
	
	JPanel panel = new JPanel();
	GridBagLayout lay = new GridBagLayout();
	Border etched = BorderFactory.createEtchedBorder();
	Border titled = BorderFactory.createTitledBorder( etched, "Resource Info");
	panel.setLayout(lay);
	panel.setBorder(titled);
	
	panel.add(resourceNameLabel, createConstraints(0,0,3,1,1,1));
	panel.add(resourceIDLabel, createConstraints(0,1,1,1,1,1));
	panel.add(resourceIDText, createConstraints(0,2,3,1,1,1));
	panel.add(resourceEntryLabel, createConstraints(0,3,1,1,1,1));
	panel.add(resourceEntryText, createConstraints(0,4,3,1,1,1));
	panel.add(resourceOrganizationLabel, createConstraints(0,5,1,1,1,1));
	panel.add(resourceOrganizationText, createConstraints(0,6,3,1,1,1));
	panel.add(resourceOrganizationLabel, createConstraints(0,7,1,1,1,1));
	panel.add(resourceOrganizationText, createConstraints(0,8,3,1,1,1));
	panel.add(resourceTitleLabel, createConstraints(0,9,1,1,1,1));
	panel.add(resourceTitleText, createConstraints(0,10,3,1,1,1));
	panel.add(resourceDateLabel, createConstraints(0,11,1,1,1,1));
	panel.add(resourceDateText, createConstraints(0,12,3,1,1,1));
	panel.add(resourceCopyrightLabel, createConstraints(0,13,1,1,1,1));
	panel.add(resourceCopyrightText, createConstraints(0,14,3,1,1,1));
	panel.add(resourceOtherAccessLabel, createConstraints(0,15,1,1,1,1));
	panel.add(resourceOtherAccessText, createConstraints(0,16,3,1,1,1));
	
	panel.add(antSpeciesLabel, createConstraints(0,17,1,1,1,1));
	panel.add(antSpeciesText, createConstraints(0,18,1,1,1,1));
	
	getContentPane().add(panel, "Center");
	
	panel = new JPanel();
	panel.add( nextButton );
	panel.add( cancelButton );
	
	getContentPane().add(panel, "South");
  }
  
  private boolean missingData()
  {
  	  ID = resourceIDText.getText().trim();
	  entry = resourceEntryText.getText().trim();
	  organization = resourceOrganizationText.getText().trim();
	  title = resourceTitleText.getText().trim();
	  date = resourceDateText.getText().trim();
	  copyright = resourceCopyrightText.getText().trim();
	  otherAccess = resourceOtherAccessText.getText().trim();
	  species = antSpeciesText.getText().trim();
	  
	  if((ID.length()==0)||(entry.length()==0)||(organization.length()==0)||(title.length()==0)||(date.length()==0)||(copyright.length()==0)||(otherAccess.length()==0)||(species.length()==0))
	  	return true;
	  return false;
  }
  
  public void actionPerformed ( ActionEvent evt )
  {
	Object src = evt.getSource();
	
	if ( src == nextButton )
	{
	  if ( missingData() )
	  {
		JOptionPane.showMessageDialog( this, "Fill in the form, please", "Anthill&Jxta: Missing data", JOptionPane.ERROR_MESSAGE);
	  }
	  else
	  {
		try
		{ 
		  AbstractForm abForm = new AbstractForm(this);
		  abForm.show();
		  LinkedList data = abForm.getData();
		  if( data != null )
		  {
		  	data = addData( data );
		  	Bib2XML b2x = new Bib2XML(data);
		  	org.w3c.dom.Document doc = b2x.doFinal();
		  	String docAsString = b2x.toString();
			long dim = file.length();
			Long lng = new Long(dim);
		    int len = lng.intValue();
			byte[] content = new byte[len];
		  	FileInputStream is = new FileInputStream(file);
		  	is.read(content);
		  	
		  	nest.shareResource( content, file.getName(), "", docAsString, species );
		  	
		  	//RequestId reqId = new jxtaimpl.ids.RequestId( nest.getNestId() );
		  	//String keys = file.getName()+" "+type+" "+desc;
		  	//nest.insRequest( reqId, keys );
		  	
		  	JOptionPane.showMessageDialog( this, "Done!", "Anthill&Jxta: Shared!", JOptionPane.INFORMATION_MESSAGE);
		  	this.dispose();
		  }
		}
		catch( SpeciesNotFoundException snfe )
		{
			JOptionPane.showMessageDialog( this, "Wrong ant species.", "Anthill&Jxta: Wrong Data", JOptionPane.ERROR_MESSAGE);  
		}
		catch( ResourceAlreadyStoredException rase)
		{
		    JOptionPane.showMessageDialog( this, "The resource seems to be already in the storage, sirry.", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);    
		} 
		catch( CapacityException ce )
		{
			JOptionPane.showMessageDialog( this, "Not enough space available, sorry.", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
		}   
		catch( IOException ioe )
		{
			JOptionPane.showMessageDialog( this, "I/O Exception, sorry.", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
		}  
		catch( ParserConfigurationException pce )
		{
			JOptionPane.showMessageDialog( this, "Parser Exception, sorry.", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
		}
		catch( TransformerException te )
		{
			JOptionPane.showMessageDialog( this, "Transformer Exception, sorry.", "Anthill&Jxta: Error", JOptionPane.ERROR_MESSAGE);
		}
	  }
	} 
	else
	{
		this.dispose();
	}
  }
  
  private LinkedList addData(LinkedList data)
  {
  	data.add("BIB-VERSION:: TR-CS-V2.1");
	data.add("ID:: "+ID);
	data.add("ENTRY:: "+entry);
	data.add("ORGANIZATION:: "+organization);
	data.add("TITLE:: "+title);
	data.add("DATE:: "+date);
	data.add("COPYRIGHT:: "+copyright);
	data.add("OTHER_ACCESS:: "+otherAccess);
	
	return data;
  }
  
  private GridBagConstraints createConstraints(int gx, int gy, int gw, int gh, int wx, int wy)
	{
	GridBagConstraints c = new GridBagConstraints();

	c.gridx = gx;
	c.gridy = gy;
	c.gridwidth = gw;
	c.gridheight = gh;
	c.weightx = wx;
	c.weighty = wy;
	 c.insets = new Insets(1,1,1,1);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
  
	return c;
	}
}
